#ifndef SERIAL_FLASH_H_
#define SERIAL_FLASH_H

/*************************************************************************************************************************
									File Details
**************************************************************************************************************************
	File name 							: Serial_FLASH.h

	Purpose								: This file has include file, macros, and function declaration corresponding to
										  Serial_FLASH.C file

	Author								: Mitesh Moonat & Harshit Gaharwar

	VDSP version used for testing		: VisualDSP++5.0, update 10

	Hardware used						: ADSP-21469, ADSP-21479, ADSP-21489, ADSP-21375, ADSP-21369, ADSP-21364,
										  ADSP-21262  EZ-Kits

	Connection details 					: See "main.C"

	Guidelines for change				: If the FLASH device connected doesn't support JEDEC READ ID (0x9F) command,
										  make sure that the default parameters for the FLASH device are defined in this
										  file and make sure to comment/uncomment the macros "DEFAULT_<flash device>"
										  to select one out of multiple default values
**************************************************************************************************************************/

/*************************************************************************************************************************
									Include Files
**************************************************************************************************************************/
#include "SPIInit.h"
#include "math.h"
#include "string.h"
#include "stdio.h"
#include "stdlib.h"

/*************************************************************************************************************************
									Structures
**************************************************************************************************************************/
typedef struct
{
	char cManID; 		//Manufacturer ID
	short sDeviceID; 	//Device ID

} s_id;



typedef struct
{
	unsigned int uiBytesPerPage;
	unsigned int uiPagesPerSector;
	unsigned int uiTotalSectors;
	unsigned int uiTotalBytes;

} s_sector_map_info;


typedef struct
{

	unsigned char	WREN;
	unsigned char 	RDID;
	unsigned char 	RDSR;
	unsigned char 	READ;
	unsigned char 	PP;
	unsigned char 	SE ;

} s_command_set;


typedef struct
{

	s_id 					IDInfo;
	s_sector_map_info		SectorInfo;
	s_command_set			CommandSet;
	char					ManName[20];		//Manufaturer name
	char					DeviceName[100];		//Device name


} SERIAL_FLASH_DEVICE_INFO;


/*************************************************************************************************************************
									Global variables defined as extern
**************************************************************************************************************************/
extern float g_fSPICLK;
extern SERIAL_FLASH_DEVICE_INFO g_serial_flash_device_info;


/*************************************************************************************************************************
									MACROs
*************************************************************************************************************************/

#define LDR_TEMP_BUFF_SIZE 4096

#define DEFAULT_AT25F2048
//#define DEFAULT_SST25LF020A


#ifdef DEFAULT_AT25F2048

	//Default definitions for AT25F2048
	#define WREN_DEFAULT 				0x06	//Write Enable
	#define RDSR_DEFAULT 				0x05   //Read Status Register
	#define READ_DEFAULT 				0x03   //Read Data Bytes (f <= 20 MHz)
	#define PP_DEFAULT					0x02   //Page Program
	#define SE_DEFAULT  				0x20   //Sector Erase				--> Different

	#define MANUFATURER_DEFAULT			"Atmel"
	#define DEVICE_NAME_DEFAULT			"2Mbit High Speed SPI Serial Flash Memory"

	#define BYTES_PER_PAGE_DEFAULT		256
	#define PAGES_PER_SECTOR_DEFAULT	256
	#define TOTAL_SECTORS_DEFAULT		4
	#define TOTAL_BYTES_DEFAULT			262144

#endif


#ifdef DEFAULT_SST25LF020A

	//Default definitions for SST25LF020A
	#define WREN_DEFAULT 				0x06	//Write Enable
	#define RDSR_DEFAULT 				0x05   //Read Status Register
	#define READ_DEFAULT 				0x03   //Read Data Bytes (f <= 20 MHz)
	#define PP_DEFAULT					0x02   //Page Program
	#define SE_DEFAULT  				0x52   //Sector Erase				--> Different

	#define MANUFATURER_DEFAULT			"SST"
	#define DEVICE_NAME_DEFAULT			"2 Mbit SPI Serial Flash"

	#define BYTES_PER_PAGE_DEFAULT		256
	#define PAGES_PER_SECTOR_DEFAULT	16			//4 KB sectors
	#define TOTAL_SECTORS_DEFAULT		64			//8 32 KB blocks - 8*8 = 64 sectors
	#define TOTAL_BYTES_DEFAULT			262144

#endif

//Status Register bits
#define WIP 		0x01	//Write in progress
#define WEL			0x02	//Write Enable Latch
#define BP0			0x04
#define BP1			0x08
#define BP2			0x10
#define SRWD		0x80

/*************************************************************************************************************************
									Function Declarations
**************************************************************************************************************************/
//Device Identification related function
void Serial_FLASH_Read_Identification(s_id* DEVICE_INFO);
void Serial_FLASH_Collect_Device_Info(SERIAL_FLASH_DEVICE_INFO* serial_flash_device_info);
void Serial_FLASH_Print_Device_Info(SERIAL_FLASH_DEVICE_INFO* serial_flash_device_info);

//Status read function
unsigned char Serial_FLASH_Read_Status(SERIAL_FLASH_DEVICE_INFO *Serial_FLASH_Info);

//Write enable function
bool Serial_FLASH_Write_Enable(SERIAL_FLASH_DEVICE_INFO *Serial_FLASH_Info);

//Erase
void Serial_FLASH_Sector_Erase(unsigned int sector_address, SERIAL_FLASH_DEVICE_INFO *Serial_FLASH_Info);

//Read
unsigned char Serial_FLASH_Read_Byte(unsigned int uiAddress, bool bMSBF, SERIAL_FLASH_DEVICE_INFO *Serial_FLASH_Info);
void Serial_FLASH_Read_Bytes(unsigned char* cBuff, unsigned int uiStartAddress, unsigned int uiLength, bool bMSBF, SERIAL_FLASH_DEVICE_INFO *Serial_FLASH_Info);

//Write
void Serial_FLASH_Page_Program(unsigned char *ucBuff, unsigned int uiStartAddress, unsigned int uiLength, bool bMSBF, SERIAL_FLASH_DEVICE_INFO *Serial_FLASH_Info );
void Serial_FLASH_Write_Byte(unsigned int uiAddress, unsigned char data, bool bMSBF, SERIAL_FLASH_DEVICE_INFO *Serial_FLASH_Info );
void Serial_FLASH_Write_Bytes(unsigned char* ucBuff, unsigned int uiStartAddress, unsigned int uiLength, bool bMSBF, SERIAL_FLASH_DEVICE_INFO *Serial_FLASH_Info);


//Programming ldr file
#if !defined(__2137x__)
	int Program_LDR_File_to_Serial_FLASH(char* sLDRFilePath, unsigned int uiStartAddress, SERIAL_FLASH_DEVICE_INFO *Serial_FLASH_Info);
#endif

int Program_LDR_File_to_Serial_FLASH_From_Memory(unsigned char* ucpSourceBuff, unsigned int uiLdrLength, unsigned int uiStartAddress, SERIAL_FLASH_DEVICE_INFO *Serial_FLASH_Info);

/************************************************************************************************************************/
#endif

/**************************************** END OF FILE*********************************************************************/
